__title__ = 'Djangondor'
__version__ = '1.8.1'
__author__ = 'Dalitso Sakala <dalitso.1sc@gmail.com>'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2024-2025'

# Version synonym
VERSION = __version__
